/* OscThread -- thread for making an oscillator go go go
 *	this one has a filter!
 *
 * 		Brad Garton, spring 2003
 *
*/
import com.softsynth.jsyn.*;

class OscThread extends Thread
{
	SawtoothOscillator	theOsc;

	// the "SynthEnvelope" will contain the data for the envelope;
	// it is the thing that you 'queue' on the EnvelopePlayer
	SynthEnvelope		theEnv;

	// the "EnvelopePlayer" is the thing that puts out the envelope
	// values (usually 0 to 1, for fade-ups and fade-downs)
	EnvelopePlayer		theEnvPlayer;

	// this array creates the envelope -- it is listed by time, value,
	// time, value, time, value... pairs where "time" is the length
	// of time (in seconds) that it will take to reach "value"
	double [] envdata =
	{
		0.0, 0.0,
		0.5, 1.0,
		0.5, 0.0
	};

	// we will use the MultiplyUnit to multiply together the output of the
	// SynthEnvelope and the SawtoothOscilator to get the amp fade up&down
	MultiplyUnit		theMult;

	LineOut			oscOut;

	public OscThread() throws SynthException
	{
		theOsc = new SawtoothOscillator();

		// envdata has already been set up
		theEnv = new SynthEnvelope(envdata);
		theEnvPlayer = new EnvelopePlayer();

		theMult = new MultiplyUnit();
		oscOut = new LineOut();

		// connect 'em up!
		theOsc.output.connect(theMult.inputA);
		theEnvPlayer.output.connect(theMult.inputB);

		theMult.output.connect(0, oscOut.input, 0);
		theMult.output.connect(0, oscOut.input, 1);

		theOsc.amplitude.set(0.5);
		theOsc.frequency.set(252.0);
	}

	public void run()
	{
		try
		{
			theOsc.start();
			theEnvPlayer.start();
			theMult.start();
			oscOut.start();

			// clear()'ing and queue()'ing the SynthEnvelope
			// is how we trigger the fade up&down
			theEnvPlayer.envelopePort.clear();
			theEnvPlayer.envelopePort.queue(theEnv);
		} catch(SynthException e) {
			SynthAlert.showError(e);
		}
	}
}
